#include "sht11.h"

static int DIO, CLK, CS;

static int SHT_TEMP = 0b00011;
static int SHT_HUMI = 0b00101;
static int SHT_STAT_W = 0b00110;
static int SHT_STAT_R = 0b00111;
static int SHT_RESET = 0b11110;

static int HEATER_ON = 0b00000100;
static int HEATER_OFF = 0b00000000;

static int T_OFFSET_3V3 = 3964; 

static int ACK  = 0;
static int NACK = 1;

static int NO = 0;
static int YES = 1;

static int MOVE_TO = 2;
static int CLR_RET = 11;

static int DegSym = 186;

void sht11_pins(int sDIO, int sCLK)
{
  DIO = sDIO;
  CLK = sCLK;
  low(CLK);
  high(DIO);
  for(int i = 0; i < 9; i++)     
  {
    high(CLK);
    low(CLK);
  }
}

void sht11_transmitStartSeq(void)
{
  input(DIO);
  low(CLK);
  high(CLK);
  low(DIO);
  low(CLK);
  high(CLK);
  input(DIO);
  low(CLK);
}
int sht11_tempRaw(void)
{
  sht11_transmitStartSeq();
  sht11_writeByte(SHT_TEMP);
  sht11_wait();
  int soT = sht11_readByte(ACK) << 8;
  soT |= sht11_readByte(NACK); 
  return soT;
}

float sht11_degC(void)
{
  float soT = (float) sht11_tempRaw();
  float sTC = (((float) soT) * 0.01) - 39.636; 
  return sTC;
}

float sht11_degF(void)
{
  float soT = (float) sht11_tempRaw();
  //float sTf = (((float) soT) * 0.018)-39.322;
  float sTf = (((float) soT) * 0.018) - 44.922; // Adjusted to another temp.
  return sTf;
}

int sht11_rhRaw(void)
{
  sht11_transmitStartSeq();
  sht11_writeByte(SHT_HUMI);
  sht11_wait();
  int soRH = sht11_readByte(ACK) << 8;
  soRH |= sht11_readByte(NACK);
  return soRH;
}

float sht11_rhLinear(void)
{
  //sRH = -4 + (0.0405 * soRH) + -0.0000028 * soRH^2
  //float sRHL = -4.0 + (0.0405 * (float) soRH) + 
  //            (-0.0000028 * (float) soRH * (float) soRH);
  float soRH = (float) sht11_rhRaw();
  float sRHL = -4.0 + (0.0405 * (float) soRH) + 
              (-0.0000028 * (float) soRH * (float) soRH);
  return sRHL;
}

float sht11_rhTrue(void)
{
  //sRHT = (sTC - 25) * (0.01 + (0.00008*soRH) + sRHL;
  float sTC = (float) sht11_degC();
  float soRH = (float) sht11_rhRaw();
  float sRHL = (float) sht11_rhLinear();
  float sRHT = ((sTC - 25.0) * (0.01 + (0.000028 * (float)soRH))) + sRHL;
  return sRHT;
}

float sht11_dewPointC(void)
{
  // This one is from SenserionDemo.spin by Cam Thompson.
  //h = (log10(rh) - 2.0) / 0.4343 + (17.62 * t) / (243.12 + t)
  float sRHT = sht11_rhTrue();
  float sTC = sht11_degC();

  float h = ((log10f(sRHT)-2.0) / 0.4343) + ((17.62 * sTC) /
            (243.12 + sTC));
  float sDPC = (243.12 * h) / (17.62 - h);
  return sDPC;   
} 

int sht11_writeByte(char byte)
{
  shift_out(DIO, CLK, MSBFIRST, 8, byte);
  int ackbit = shift_in(DIO, CLK, LSBPRE, 1);
  return ackbit;
}

void sht11_wait(void)
{
  input(DIO);
  for(int i = 0; i <  300; i++)
  {
    if(input(DIO) == 0) break;
    pause(1);
  }
}

char sht11_readByte(char ackBit)
{
  char shtData = shift_in(DIO, CLK, MSBPRE, 8);
  shift_out(DIO, CLK, LSBFIRST, 1, ackBit);
  input(DIO);
  return shtData;
}

void sht11_writeStatus(char statusVal)
{
  sht11_transmitStartSeq();
  sht11_writeByte(SHT_STAT_W);
  sht11_writeByte(statusVal);
}

